package simple

import (
	"os"
	"fmt"
	"github.com/blakesmith/ar"
	"bytes"
	"io"
	"compress/gzip"
	"archive/tar"
	"log"
	"path/filepath"
	"io/ioutil"
	"strings"
	"crypto/md5"
	"crypto/sha1"
	"crypto/sha256"
	"encoding/hex"
	"golang.org/x/crypto/openpgp"
	"time"
	"meow.tf/deb-simple/deb/release"
)

func inspectPackage(filename string) (string, error) {
	f, err := os.Open(filename)
	if err != nil {
		return "", fmt.Errorf("error opening package file %s: %s", filename, err)
	}

	arReader := ar.NewReader(f)
	defer f.Close()
	var controlBuf bytes.Buffer

	for {
		header, err := arReader.Next()

		if err == io.EOF {
			break
		}

		if err != nil {
			return "", fmt.Errorf("error in inspectPackage loop: %s", err)
		}

		if strings.Trim(header.Name, "/") == "control.tar.gz" {
			io.Copy(&controlBuf, arReader)
			return inspectPackageControl(controlBuf)
		}
	}
	return "", nil
}

func inspectPackageControl(filename bytes.Buffer) (string, error) {
	gzf, err := gzip.NewReader(bytes.NewReader(filename.Bytes()))
	if err != nil {
		return "", fmt.Errorf("error creating gzip reader: %s", err)
	}

	tarReader := tar.NewReader(gzf)
	var controlBuf bytes.Buffer
	for {
		header, err := tarReader.Next()

		if err == io.EOF {
			break
		}

		if err != nil {
			return "", fmt.Errorf("failed to inspect package: %s", err)
		}

		name := header.Name

		switch header.Typeflag {
		case tar.TypeDir:
			continue
		case tar.TypeReg:
			if name == "./control" {
				io.Copy(&controlBuf, tarReader)
				return controlBuf.String(), nil
			}
		default:
			log.Printf(
				"Unable to figure out type : %c in file %s\n",
				header.Typeflag, name,
			)
		}
	}
	return "", nil
}


func createRelease(config Conf, distro string) error {
	outfile, err := os.Create(filepath.Join(config.DistPath(distro), "Release"))

	if err != nil {
		return fmt.Errorf("failed to create Release: %s", err)
	}

	defer outfile.Close()

	r := &release.Release{
		Suite: distro,
		Architectures: config.Repo.ArchitectureNames(),
		Components: []string{"main"},
		Date: time.Now(),
	}

	for _, arch := range config.Repo.ArchitectureNames() {
		absolutePath := filepath.Join(config.DistPath(distro), "main", "binary-" + arch)

		list, err := ioutil.ReadDir(absolutePath)

		if err != nil {
			continue
		}

		basePath := filepath.Join("main", "binary-" + arch)

		for _, file := range list {
			filePath := filepath.Join(absolutePath, file.Name())

			fileLocalPath := filepath.Join(basePath, file.Name())

			fileLocalPath = strings.Replace(fileLocalPath, "\\", "/", -1)

			f, err := os.Open(filePath)

			if err != nil {
				return err
			}

			var size int64 = file.Size()

			if size == 0 {
				if stat, err := os.Stat(filePath); err == nil {
					size = stat.Size()
				}
			}

			var (
				md5hash    = md5.New()
				sha1hash   = sha1.New()
				sha256hash = sha256.New()
			)

			w := io.MultiWriter(md5hash, sha1hash, sha256hash)

			io.Copy(w, f)

			repoFile := &release.File{
				Name: fileLocalPath,
				Size: size,
				MD5Sum: hex.EncodeToString(md5hash.Sum(nil)),
				SHA1: hex.EncodeToString(sha1hash.Sum(nil)),
				SHA256: hex.EncodeToString(sha256hash.Sum(nil)),
			}

			r.Append(repoFile)

			f.Close()
			f = nil
		}
	}

	r.Write(outfile)

	if pgpEntity != nil {
		return signRelease(config, distro)
	}

	return nil
}

func signRelease(config Conf, distro string) error {
	distPath := config.DistPath(distro)

	f, err := os.Open(filepath.Join(distPath, "Release"))

	if err != nil {
		return fmt.Errorf("failed to read Release: %s", err)
	}

	defer f.Close()

	gpgfile, err := os.Create(filepath.Join(distPath, "Release.gpg"))

	if err != nil {
		return fmt.Errorf("failed to create Release.gpg: %s", err)
	}

	defer gpgfile.Close()

	if err := openpgp.ArmoredDetachSignText(gpgfile, pgpEntity, f, nil); err != nil {
		return fmt.Errorf("failed to sign Release: %s", err)
	}

	return nil
}